/* N-body calculation using Greenspan 1st-order discrete mechanics.    */
/* Incorporates xyz-parallelism, uses domain pointers, combines        */
/* particle and pair domains, diagonal processors store particles      */
/* Spreading and accumulation coded in PARIS, more globals,            */
/* Booleans stored in char fields.                                     */
/* PARIS recoded to V5 standards				       */

/* *********************  Only for checking addresses  *************** */

/*    Provides for insertion of asteroids uniformly spaced in circular */
/*    orbit of specified radius, control over printing.                */

#include <stdio.hs>
#include <cm/cm.hs>
#include <math.hs>
#include "cmtime.hs"

/* Max number of bodies in the calculation.  N = 64 fits in 16k procs
   N must be a power of 2, 4N^2 <= no. of v. procs.  LLN = log2(N)    */

#define N  64
#define LN  6

int NB;      /* Number of bodies in system, <= N */
int LNB;     /* Largest power of 2 < NB */
int LLNB;    /* log2(LNB) */
float G;     /* Gravitational constant */

#define TWOPI 6.2831853

int PX;      /* Print control */
int NX;      /* Number of bodies given explicitly */
int NA;      /* Number of asteroids */
float AR;    /* Asteroid orbit radius */

/* Pair domain:								*/
/* Particles, and pairs of particles.  Diagonal entries have particle	*/
/* states.  Some variables could be overloaded. 			*/
/* act  - processor is active pair					*/
/* here - processor represents particle					*/
/* sp   - processor participates in spreading                           */
/* smk  - control for spreading, accumulation, when procs change        */
/* psym - plot symbol							*/
/* self - self-address for manipulation                                 */

/* VT-100 plotting for N-body programs - Capital letters leave trails	*/

/* Maximum screen size */

#define SXSZ 45
#define SYSZ 90

int SX=22, SY=80;	/* Actual size, default initialization */
/* SX should leave room below display for time line followed by NL */

/* The order of the next four declarations is essential to obtain correct
   cube addresses for pair.  */

domain screen { int sx, sy;  char oldc, newc, chng; };

domain pair { int c, i, j, smk;  char act, sm, here, sp, smm, psym;
	      CM_cubeaddr_t self;
	      float x0, x1, v0, v1, m, d0, r0, ac, w;
	      domain pair *r; };

domain screen Sc[SXSZ][SYSZ];

domain pair P[3][N][N];

float XL, XR, YB, YT;  /* Screen limits for physical coordinates */
int scgood;            /* Display assumed to correspond to Sc    */

/* Cm addresses for PARIS code */

CM_memaddr_t smk_addr, sp_addr, smm_addr, self_addr, ac_addr, w_addr;


/* Pairs will be represented symmetrically.  The function setup
   (below) initializes these so that pi, pj point to appropriate
   particle in corresponding coordinate.   */

void pair::setup()
 { int offset;  mono int k;
   for ( LNB = 1, k = 0;  LNB < NB;  LNB <<= 1, k++ ) ;
   sp = (i<LNB) && (j < LNB);  LNB >>= 1;  LLNB = k-1;
   self = (CM_cubeaddr_t)this;
   smk_addr = &smk;  smm_addr = &smm;  sp_addr = &sp;
   self_addr = &self;  ac_addr = &ac;  w_addr = &w;
   offset = (this - &P[0][0][0]);
   c = offset/(N*N);  i = (offset/N)%N;  j = offset%N;
   sm = (i < NB) && (j < NB);
   act = sm && (i != j);  here = sm && (i == j);
   r = (c<2) ? this+N*N : this-2*N*N;
   smm = 1;  smk = i^j;  /* xor */
   for ( k = 1; k < NB; k <<= 1 )
    if (smm && (k & smk)) { smk = k;  smm = 0; }
   if ( !sp ) smk = 0;
 }


void main()
 { NB = 128;
   [domain pair].{ setup(); }
   printf("pair[0,0,0] at %d, pair[1,0,0] at %d, pair[2,0,0] at %d\n",
       (int)P[0][0][0].this, (int)P[1][0][0].this, (int)P[2][0][0].this);
 }